<?php

define('SmsApiUrl', 'https://api.sendcloud.net/smsapi/send');

function sign(string $apiKey, array $params)
{
    ksort($params);
    $signParts = [$apiKey, $apiKey];
    foreach ($params as $key => $value) {
        array_splice($signParts, -1, 0, $key.'='.$value);
    }

    return md5(join('&', $signParts));
}

/*
 * msgType:
 *   0 表示短信，1 表示彩信，2 表示国际短信，3 表示国内语音，5 表示影音。
 *   默认值为 0。
 */
function sendsms(
    string $apiUser,
    string $apiKey,
    string $phone,
    int $templateId,
    array $templateParams,
    int $msgType = 0
) {
    $params = [
        'smsUser' => $apiUser,
        'templateId' => $templateId,
        'msgType' => $msgType,
        'phone' => $phone,
        'vars' => json_encode($templateParams, 1),
    ];
    $params['signature'] = sign($apiKey, $params);
    // https://www.php.net/manual/zh/context.http.php
    $context = stream_context_create([
        'http' => [
            'method' => 'POST',
            'header' => 'Content-Type:application/x-www-form-urlencoded',
            'content' => http_build_query($params),
            'timeout' => 30,
        ],
    ]);
    $result = file_get_contents(SmsApiUrl, FILE_TEXT, $context);

    return json_decode($result, true);
}

function sendsms_curl(
    string $apiUser,
    string $apiKey,
    string $phone,
    int $templateId,
    array $templateParams,
    int $msgType = 0
) {
    $params = [
        'smsUser' => $apiUser,
        'templateId' => $templateId,
        'msgType' => $msgType,
        'phone' => $phone,
        'vars' => json_encode($templateParams, 1),
    ];
    $params['signature'] = sign($apiKey, $params);

    $ch = curl_init();

    try {
        // https://www.php.net/manual/zh/function.curl-setopt.php
        curl_setopt_array($ch, [
            CURLOPT_URL => SmsApiUrl,
            CURLOPT_POST => true,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CONNECTTIMEOUT => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_POSTFIELDS => http_build_query($params),
        ]);
        $result = curl_exec($ch);

        $error = curl_error($ch);
        if ($error) {
            echo 'CurlError: '.$error."\n";

            return null;
        }
    } catch (Exception $error) {
        echo $error."\n";

        return null;
    } finally {
        curl_close($ch);
    }

    return json_decode($result, true);
}

// ################################### TEST ###################################

$apiUser = '';
$apiKey = '';
$phone = '';
$templateId = 40438;

# 方法 1（需开启 allow_url_fopen 设置）：
$vars = ['code' => (string) mt_rand(1000, 9999)];
var_dump(sendsms($apiUser, $apiKey, $phone, $templateId, $vars));

# 方法 2（推荐！需启用 curl 模块）：
$vars = ['code' => (string) mt_rand(1000, 9999)];
var_dump(sendsms_curl($apiUser, $apiKey, $phone, $templateId, $vars));

echo "Finished!!!\n";
?>